﻿Imports System.Globalization
Imports System.Speech.Synthesis
Imports System.Text.RegularExpressions
Imports System.IO
Imports System.Windows.Forms.DataVisualization.Charting

Public Class Form1

    Private rand As New Random()
    Private synthesizer As New SpeechSynthesizer()

    ' Memory: Text, Timestamp, IsEmotional, Category
    Private memory As New List(Of (Text As String, Time As DateTime, IsEmotional As Boolean, Category As String))
    Private userName As String = ""
    Private nameFilePath As String = "username.txt"
    Private logFilePath As String = "conversation_log.txt"
    Private moodTally As New Dictionary(Of String, Integer)()

    ' Emotional keywords
    Private emotionKeywords As Dictionary(Of String, String) = New Dictionary(Of String, String) From {
        {"sad", "sadness"},
        {"unhappy", "sadness"},
        {"depressed", "sadness"},
        {"anxious", "anxiety"},
        {"angry", "anger"},
        {"mad", "anger"},
        {"lonely", "loneliness"},
        {"afraid", "fear"},
        {"scared", "fear"},
        {"worried", "anxiety"},
        {"hurt", "pain"},
        {"stressed", "stress"},
        {"happy", "happiness"},
        {"excited", "excitement"}
    }

    Private responses As New Dictionary(Of String, String()) From {
        {"My name is (.*)", New String() {
            "Nice to meet you, {0}.",
            "Thank you for telling me your name, {0}.",
            "I'll remember that, {0}.",
            "Hello, {0}. It's a pleasure."
        }},
        {"I feel (.*)", New String() {
            "Why do you feel {0}?",
            "What makes you feel {0}?",
            "How long have you been feeling {0}?"
        }},
        {"I need (.*)", New String() {
            "Why do you need {0}?",
            "Would it really help you to get {0}?",
            "Are you sure you need {0}?"
        }},
        {"I'm (.*)", New String() {
            "How long have you been {0}?",
            "Do you believe it's normal to be {0}?",
            "Why do you say you're {0}?"
        }},
        {"I am (.*)", New String() {
            "How does being {0} affect you?",
            "Do you enjoy being {0}?",
            "What does being {0} mean to you?",
            "What makes you say you're {0}?"
        }},
        {"I have (.*)", New String() {
            "How long have you had {0}?",
            "What do you think caused {0}?"
        }},
        {"Because (.*)", New String() {
            "Is that the real reason?",
            "What other reasons come to mind?"
        }},
        {"I can't (.*)", New String() {
            "What makes you think you can't {0}?",
            "Have you tried to {0}?"
        }},
        {"I (.*) my (mom|dad|sister|brother|friend|partner|husband|wife)", New String() {
            "Tell me more about your relationship with your {1}.",
            "Why do you think you {0} your {1}?",
            "What has your {1} done to make you feel that way?"
        }},
        {"Hello", New String() {
            "Hello there! How can I help you today?",
            "Hi! What's on your mind today?"}},
        {"Hi", New String() {
            "Hi! How are you feeling today?",
            "Hello! I'm here to listen."}},
        {"I (.*) you", New String() {
            "Why do you {0} me?",
            "Can you explain why you {0} me?"}},
        {"You (.*) me", New String() {
            "What makes you think I {0} you?",
            "Why do you believe I {0} you?"
        }},
        {"(.*)\?", New String() {
            "Why do you ask that?",
            "What are your thoughts on that?",
            "Does that question interest you?"
        }},
        {"(.*)", New String() {
            "Please tell me more.",
            "Let's change focus a bit... Tell me about your family.",
            "Can you elaborate on that?",
            "Why do you say that?",
            "I see.",
            "Very interesting.",
            "I understand."
        }}
    }

    Private Sub btnSend_Click(sender As Object, e As EventArgs) Handles btnSend.Click
        Dim userInput As String = txtInput.Text.Trim()
        If userInput = "" Then Return

        txtOutput.AppendText("You: " & userInput & Environment.NewLine)
        AppendToLog("You: " & userInput)

        Dim response As String = GetResponse(userInput)
        txtOutput.AppendText("Eliza: " & response & Environment.NewLine & Environment.NewLine)
        AppendToLog("Eliza: " & response)

        synthesizer.SpeakAsync(response)
        txtInput.Clear()
    End Sub

    Private Sub AppendToLog(line As String)
        IO.File.AppendAllText(logFilePath, $"[{DateTime.Now:HH:mm}] {line}{Environment.NewLine}")
    End Sub

    Private Sub Form1_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        If IO.File.Exists(nameFilePath) Then
            userName = IO.File.ReadAllText(nameFilePath).Trim()
            If userName <> "" Then
                Dim greeting As String = $"Welcome back, {userName}. How are you feeling today?"
                txtOutput.AppendText("Eliza: " & greeting & Environment.NewLine & Environment.NewLine)
                synthesizer.SpeakAsync(greeting)
            End If
        End If
    End Sub

    Private Function GetResponse(input As String) As String
        input = input.Trim()

        ' Name reset
        If Regex.IsMatch(input, "reset my name", RegexOptions.IgnoreCase) Then
            userName = ""
            IO.File.Delete(nameFilePath)
            Return "Okay, I've forgotten your name. What should I call you now?"
        End If

        If Regex.IsMatch(input, "change my name to (.+)", RegexOptions.IgnoreCase) Then
            Dim newName = Regex.Match(input, "change my name to (.+)", RegexOptions.IgnoreCase).Groups(1).Value
            userName = CultureInfo.CurrentCulture.TextInfo.ToTitleCase(newName.ToLower())
            IO.File.WriteAllText(nameFilePath, userName)
            Return $"Got it. I'll call you {userName} from now on."
        End If

        ' Fade old memory
        memory = memory.Where(Function(m) m.IsEmotional OrElse DateDiff(DateInterval.Minute, m.Time, DateTime.Now) < 10).ToList()

        If Regex.IsMatch(input, "show mood chart", RegexOptions.IgnoreCase) Then
            ShowMoodChart()
            Return "Here is your mood chart."
        End If

        ' Mood summary request
        If Regex.IsMatch(input, "how (am I|do I) feel", RegexOptions.IgnoreCase) OrElse input.ToLower().Contains("mood") Then
            If moodTally.Count = 0 Then Return "I haven’t picked up on any moods yet."
            Dim topMood = moodTally.OrderByDescending(Function(kv) kv.Value).First().Key
            Dim emoji = GetEmojiForMood(topMood)
            Return $"You've seemed mostly {topMood} so far. {emoji}"
        End If

        If Regex.IsMatch(input, "export chat", RegexOptions.IgnoreCase) Then
            Dim exportPath = "chat_export.html"
            Dim html = "<html><body><h2>Eliza Conversation Log</h2><pre>" & File.ReadAllText(logFilePath) & "</pre></body></html>"
            File.WriteAllText(exportPath, html)
            Return "Chat exported to HTML file."
        End If

        For Each pattern In responses.Keys
            If pattern = "(.*)" AndAlso rand.Next(10) < 5 Then
                Exit For
            End If
            Dim match As Match = Regex.Match(input, pattern, RegexOptions.IgnoreCase)
            If match.Success Then
                Dim responseArray = responses(pattern)
                Dim template = responseArray(rand.Next(responseArray.Length))

                Dim captured = If(match.Groups.Count > 1, match.Groups(1).Value, "")
                Dim secondCaptured = If(match.Groups.Count > 2, match.Groups(2).Value, "")

                ' Store name
                If pattern.StartsWith("My name is") AndAlso captured <> "" Then
                    userName = CultureInfo.CurrentCulture.TextInfo.ToTitleCase(captured.ToLower())
                    IO.File.WriteAllText(nameFilePath, userName)
                End If

                ' Detect emotion
                Dim lowerInput = captured.ToLower()
                Dim matchedEmotion = emotionKeywords.FirstOrDefault(Function(kv) lowerInput.Contains(kv.Key)).Value
                Dim isEmotional = matchedEmotion IsNot Nothing

                If isEmotional Then
                    If Not moodTally.ContainsKey(matchedEmotion) Then moodTally(matchedEmotion) = 0
                    moodTally(matchedEmotion) += 1
                End If

                ' Categorize
                Dim category As String = If(matchedEmotion IsNot Nothing, matchedEmotion, If(secondCaptured <> "", secondCaptured, "general"))

                ' Store memory
                If captured <> "" Then
                    memory.Add((captured, DateTime.Now, isEmotional, category))
                End If

                ' Build reply
                Dim responseText = template.Replace("{0}", captured).Replace("{1}", secondCaptured)
                If userName <> "" AndAlso rand.Next(3) = 0 Then
                    responseText &= " " & userName & "."
                End If
                Return responseText
            End If
        Next

        ' Fallback with memory
        If memory.Count > 0 Then
            Dim emotional = memory.Where(Function(m) m.IsEmotional).ToList()
            Dim chosen = If(emotional.Any(), emotional(rand.Next(emotional.Count)), memory(rand.Next(memory.Count)))

            Dim empathyTemplates As String() = {
            "You once mentioned '{0}'—does that still affect you?",
            "Earlier you talked about '{0}'. Can we revisit that?",
            "I remember you said something about '{0}'. How is that going?"
        }

            Dim template = empathyTemplates(rand.Next(empathyTemplates.Length))
            If userName <> "" AndAlso rand.Next(3) = 0 Then
                template = userName & ", " & template
            End If
            Return String.Format(template, chosen.Text)
        End If

        Return If(userName <> "" AndAlso rand.Next(3) = 0,
              "I'm not sure I understand you fully, " & userName & ".",
              "I'm not sure I understand you fully.")
    End Function

    Private Function GetEmojiForMood(mood As String) As String
        Select Case mood.ToLower()
            Case "sadness" : Return "😢"
            Case "anxiety" : Return "😰"
            Case "anger" : Return "😠"
            Case "loneliness" : Return "😔"
            Case "fear" : Return "😨"
            Case "stress" : Return "😫"
            Case "happiness" : Return "😊"
            Case "excitement" : Return "😄"
            Case Else : Return "🫥"
        End Select
    End Function

    Private Sub ShowMoodChart()
        Dim chart As New DataVisualization.Charting.Chart
        chart.Size = New Drawing.Size(400, 300)
        chart.ChartAreas.Add(New ChartArea("Main"))

        Dim series As New Series("Mood")
        series.ChartType = SeriesChartType.Column
        series.IsValueShownAsLabel = True

        For Each kvp In moodTally
            series.Points.AddXY(kvp.Key, kvp.Value)
        Next

        chart.Series.Add(series)
        Dim chartForm As New Form()
        chartForm.Text = "Mood Chart"
        chartForm.Controls.Add(chart)
        chart.Dock = DockStyle.Fill
        chartForm.Show()
    End Sub

End Class
